# pylint: disable=missing-function-docstring
import datetime
import json
import logging
import os
import traceback

from azure.appconfiguration import AzureAppConfigurationClient, ConfigurationSetting
from azure.identity import DefaultAzureCredential
from azure.keyvault.secrets import SecretClient
from texttable import Texttable


def is_json(variable):
    try:
        _ = json.loads(variable)
        return True
    except (ValueError, TypeError):
        return False




def except_block(exception: Exception, exception_message="No message"):
    """
    Log the details of an exception, including module name, function name, custom message,
    time of occurrence, exception type, arguments, instance, stack trace, and local variables.

    :param exception: The exception instance.
    :type exception: Exception
    :param exception_message: A custom message for the exception, defaults to "No message".
    :type exception_message: str, optional
    """
    exception_timne = str(datetime.datetime.now())
    exception_type = str(type(exception))
    exception_argument = str(exception.args)
    exception_instance = str(exception)

    stack_trace = traceback.format_exc()
    logging.log(
        msg="Exception message: {message}".format(message=exception_message),
        level=logging.ERROR,
    )
    logging.log(
        msg="Exception occured at: {message}".format(message=exception_timne),
        level=logging.ERROR,
    )
    logging.log(
        msg="Exception instance type: {message}".format(message=exception_type),
        level=logging.ERROR,
    )
    logging.log(
        msg="Exception argument: {message}".format(message=exception_argument),
        level=logging.ERROR,
    )
    logging.log(
        msg="Exception instance: {message}".format(message=exception_instance),
        level=logging.ERROR,
    )
    logging.log(
        msg="Exception stack trace: {message}".format(message=stack_trace),
        level=logging.ERROR,
    )


def _keyVaultAccess(key):
    try:
        key_vault_uri = str(key["uri"]).split("/secrets/")[0]
        secret_name = str(key["uri"]).split("/secrets/")[1]
        credential = DefaultAzureCredential(
            exclude_interactive_browser_credential=False
        )
        client = SecretClient(vault_url=key_vault_uri, credential=credential)
        retrieved_secret = client.get_secret(secret_name)
    except Exception as e:
        exception_message = "KeyVault access failed"
        except_block(exception=e, exception_message=exception_message)
    return retrieved_secret.value


def listConfig():
    output_table = Texttable()
    table = [
        ["key", "last_modified", "content_type", "keyVault_reference", "access_level"]
    ]
    appConfigurationConnectionString = os.environ["APPCONFIGURATION_CONNECTION_STRING"]
    app_config_client = AzureAppConfigurationClient.from_connection_string(
        appConfigurationConnectionString
    )
    for config in app_config_client.list_configuration_settings():
        config = config.as_dict()
        try:
            resource_type_definition = config["tags"]["type"]
            resource_base_type = resource_type_definition.split(".")[1].split("/")[0]

            config_type = str(config["content_type"]).lower()
            if "keyvaultref" in config_type:
                is_keyvault_ref = "True"
            else:
                is_keyvault_ref = "False"
            table.append(
                [
                    config["key"],
                    config["last_modified"],
                    resource_base_type,
                    is_keyvault_ref,
                    config["read_only"],
                ]
            )
            line = "key: {key}, last_modified: {last_modified}, read_only: {read_only}".format(
                key=config["key"],
                last_modified=config["last_modified"],
                read_only=config["read_only"],
            )
            logging.info(line)
        except Exception as e:
            exception_message = "feature_flag: {feature_id}, last_modified: {last_modified}, value: {value}".format(
                feature_id=config["feature_id"],
                last_modified=config["last_modified"],
                value=str(config["value"]),
            )
            except_block(exception=e, exception_message=exception_message)
    output_table.add_rows(table)
    logging.info(output_table.draw())


def getValue(key, deactivate_kv_access=False, label=None):
    response = ""
    appConfigurationConnectionString = os.environ["APPCONFIGURATION_CONNECTION_STRING"]
    app_config_client = AzureAppConfigurationClient.from_connection_string(
        appConfigurationConnectionString
    )
    try:
        retrieved_config_setting = app_config_client.get_configuration_setting(
            key=str(key), label=label
        )
        config_type = retrieved_config_setting.as_dict()["content_type"]
        if (deactivate_kv_access == False) and ("keyvaultref" in config_type):
            var = retrieved_config_setting.value
            if is_json(variable=var):
                print("Variable is in JSON format.")
                result = json.loads(var)
            else:
                print("Continuing with string interpretation")
                result = str(var)
            response = _keyVaultAccess(key=result)
        else:
            response = retrieved_config_setting.value
        logging.info("Key: " + retrieved_config_setting.key + ", Value: " + response)
    except:
        for config in app_config_client.list_configuration_settings():
            retrieved_config_setting = config.as_dict()
            if "feature_id" in retrieved_config_setting.keys():
                if retrieved_config_setting["feature_id"] == key:
                    logging.info(
                        "Key: "
                        + retrieved_config_setting["feature_id"]
                        + ", Value: "
                        + retrieved_config_setting["value"]
                    )
                    var = retrieved_config_setting["value"]
                    if is_json(variable=var):
                        print("Variable is in JSON format.")
                        result = json.loads(var)["enabled"]
                    else:
                        print("Continuing with string interpretation")
                        result = str(var)
                    response = result
    return response


def setValue(
    key,
    value=None,
    content_type="charset=utf-8",
    tags: dict = {},
    label=None,
    deactivate_kv_access=False,
):
    config_setting = ConfigurationSetting(
        key=key, label=label, value=value, content_type=content_type, tags=tags
    )
    response = ""
    appConfigurationConnectionString = os.environ["APPCONFIGURATION_CONNECTION_STRING"]
    app_config_client = AzureAppConfigurationClient.from_connection_string(
        appConfigurationConnectionString
    )
    try:
        retrieved_config_setting = app_config_client.get_configuration_setting(
            key=str(key), label=label
        )
        config_type = retrieved_config_setting.as_dict()["content_type"]
        if (deactivate_kv_access == False) and ("keyvaultref" in config_type):
            var = retrieved_config_setting.value
            if is_json(variable=var):
                print("Variable is in JSON format.")
                result = json.loads(var)
            else:
                print("Continuing with string interpretation")
                result = str(var)
            response = _keyVaultAccess(key=result)
        else:
            response = retrieved_config_setting.value
        logging.info(
            "Key: "
            + retrieved_config_setting.key
            + ", Value: "
            + response
            + " already present, starting override"
        )
    except:
        response = app_config_client.set_configuration_setting(config_setting)
        logging.info("Key: " + key + ", Value: " + value + " has been set")
    return response



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.2.5'
DESCRIPTION = 'Collection of helpfull extensions'
LONG_DESCRIPTION = 'Collection of helpfull extensions'

# Setting up
setup(
    name="paddypy",
    version=VERSION,
    author="Patrik",
    author_email="<patrikhartl@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=['azure-appconfiguration>=1.1.1', 'texttable>=1.4.4', 'azure-identity>=1.5.0', 'azure-keyvault-secrets>=4.1.0', 'opencensus-ext-azure>=1.1.9'],
    keywords=['python', 'appconfiguration', 'azure'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Environment :: Web Environment"
    ]
)


import datetime
import logging
import sys
import time
import traceback
from functools import wraps

from opencensus.ext.azure.log_exporter import AzureLogHandler

from . import access


class CloudLogger:
    """
    Initialize the CloudLogger class.

    :param app_logger_name: The name of the application logger.
    :type app_logger_name: str
    :param module_name: The name of the module using the logger.
    :type module_name: str
    :param applicationinsights_key: The Application Insights key, defaults to None.
    :type applicationinsights_key: str, optional
    :param logger_level: The logging level, defaults to 10 (DEBUG).
    :type logger_level: int, optional
    """

    def __init__(
        self,
        app_logger_name,
        module_name,
        applicationinsights_key=None,
        logger_level=10,
    ):
        self.LOGGER_LEVEL = logger_level
        self.logger = self._get_logger(
            app_logger_name=app_logger_name,
            module_name=module_name,
            applicationinsights_key=applicationinsights_key,
            logger_level=self.LOGGER_LEVEL,
        )
        self.logger.setLevel(level=logger_level)

    def _get_logger(
        self, app_logger_name, module_name, applicationinsights_key, logger_level
    ):
        """
        Get the logger instance with the specified configurations.

        :param app_logger_name: The name of the application logger.
        :type app_logger_name: str
        :param module_name: The name of the module using the logger.
        :type module_name: str
        :param applicationinsights_key: The Application Insights key.
        :type applicationinsights_key: str
        :param logger_level: The logging level.
        :type logger_level: int
        :return: The configured logger instance.
        :rtype: logging.Logger
        """
        FMT = "[{levelname:^9}] {name}: {message}"
        FORMATS = {
            logging.DEBUG: FMT,
            logging.INFO: "\33[36m{fmt}\33[0m".format(fmt=FMT),
            logging.WARNING: "\33[33m{fmt}\33[0m".format(fmt=FMT),
            logging.ERROR: "\33[31m{fmt}\33[0m".format(fmt=FMT),
            logging.CRITICAL: "\33[1m\33[31m{fmt}\33[0m".format(fmt=FMT),
        }

        class CustomFormatter(logging.Formatter):
            def format(self, record):
                log_fmt = FORMATS[record.levelno]
                formatter = logging.Formatter(log_fmt, style="{")
                return formatter.format(record)

        handler = logging.StreamHandler()
        handler.setFormatter(CustomFormatter())
        logging.basicConfig(level=logger_level, handlers=[handler])

        logger = logging.getLogger(app_logger_name).getChild(module_name)
        if applicationinsights_key:
            azure_handler = AzureLogHandler(
                connection_string=str(applicationinsights_key)
            )
            azure_handler.setLevel(logger_level)
            logger.addHandler(azure_handler)
        return logger

    # def timeit(self, func):
    #     @wraps(func)
    #     def timeit_wrapper(*args, **kwargs):
    #         start_time = time.perf_counter()
    #         result = func(*args, **kwargs)
    #         end_time = time.perf_counter()
    #         total_time = end_time - start_time
    #         self.logger.log(msg=f'Function {func.__name__}{args} {kwargs} Took {total_time:.4f} seconds', level=logging.INFO)
    #         #logging.log(f'Function {func.__name__}{args} {kwargs} Took {total_time:.4f} seconds', level=logging.DEBUG)
    #         return result
    #     return timeit_wrapper

    # def timeit_silence(self, func):
    #     @wraps(func)
    #     def timeit_wrapper(*args, **kwargs):
    #         start_time = time.perf_counter()
    #         result = func(*args, **kwargs)
    #         end_time = time.perf_counter()
    #         total_time = end_time - start_time
    #         arg_types = tuple(type(arg).__name__ for arg in args)
    #         kwarg_types = {key: type(val).__name__ for key, val in kwargs.items()}
    #         log_msg = f'Function {func.__name__}{arg_types} {kwarg_types} Took {total_time:.4f} seconds'
    #         self.logger.log(msg=log_msg, level=logging.INFO)
    #         return result
    #     return timeit_wrapper

    def timeit(self, silence_args=False):
        """
        Decorator for timing a function and logging the execution time.

        :param silence_args: If True, logs the types of the arguments instead of their values, defaults to False.
        :type silence_args: bool, optional
        :return: Decorated function.
        """

        def decorator(func):
            @wraps(func)
            def timeit_wrapper(*args, **kwargs):
                start_time = time.perf_counter()
                result = func(*args, **kwargs)
                end_time = time.perf_counter()
                total_time = end_time - start_time
                if silence_args:
                    arg_types = tuple(type(arg).__name__ for arg in args)
                    kwarg_types = {
                        key: type(val).__name__ for key, val in kwargs.items()
                    }
                    log_msg = f"Function {func.__name__}{arg_types} {kwarg_types} Took {total_time:.4f} seconds"
                else:
                    log_msg = f"Function {func.__name__}{args} {kwargs} Took {total_time:.4f} seconds"

                self.logger.log(msg=log_msg, level=logging.INFO)
                return result

            return timeit_wrapper

        return decorator

    def except_block(self, exception: Exception, exception_message="No message"):
        """
        Log the details of an exception, including module name, function name, custom message,
        time of occurrence, exception type, arguments, instance, stack trace, and local variables.

        :param exception: The exception instance.
        :type exception: Exception
        :param exception_message: A custom message for the exception, defaults to "No message".
        :type exception_message: str, optional
        """
        exception_timne = str(datetime.datetime.now())
        exception_type = str(type(exception))
        exception_argument = str(exception.args)
        exception_instance = str(exception)
        current_frame = sys._getframe().f_back
        if current_frame is not None:
            module_name = current_frame.f_globals["__name__"]
            function_name = current_frame.f_code.co_name
            local_variables = current_frame.f_locals
        else:
            module_name = "unknown"
            function_name = "unknown"
            local_variables = {"local_variables": "unknown"}

        stack_trace = traceback.format_exc()
        self.logger.log(
            msg="Exception module name: {message}".format(message=module_name),
            level=logging.ERROR,
        )
        self.logger.log(
            msg="Exception function name: {message}".format(message=function_name),
            level=logging.ERROR,
        )
        self.logger.log(
            msg="Exception message: {message}".format(message=exception_message),
            level=logging.ERROR,
        )
        self.logger.log(
            msg="Exception occured at: {message}".format(message=exception_timne),
            level=logging.ERROR,
        )
        self.logger.log(
            msg="Exception instance type: {message}".format(message=exception_type),
            level=logging.ERROR,
        )
        self.logger.log(
            msg="Exception argument: {message}".format(message=exception_argument),
            level=logging.ERROR,
        )
        self.logger.log(
            msg="Exception instance: {message}".format(message=exception_instance),
            level=logging.ERROR,
        )
        self.logger.log(
            msg="Exception stack trace: {message}".format(message=stack_trace),
            level=logging.ERROR,
        )
        self.logger.log(
            msg="Exception local variables: {message}".format(message=local_variables),
            level=logging.ERROR,
        )
